@@ -3,9 +3,10 @@
 #include <RcppEigen.h>
 
 // [[Rcpp::export]]
-Eigen::MatrixXd EigenARC(Eigen::MatrixXd X, bool centralizeX = true, int cores = 2){
+Eigen::MatrixXd EigenARC(Eigen::MatrixXd X, bool centralizeX = true, int cores = 1){
   // cseweb.ucsd.edu/~saul/papers/nips09_kernel.pdf
-  Eigen::setNbThreads(cores); int p = X.cols(), n = X.rows(); 
+  if(cores!=1) Eigen::setNbThreads(cores);
+  int p = X.cols(), n = X.rows(); 
   double tmp, Npi=3.1416, theta, J1, Kij, Norm;
   if(centralizeX){
     for(int i=0; i<p; i++){
@@ -23,8 +24,8 @@ Eigen::MatrixXd EigenARC(Eigen::MatrixXd X, bool centralizeX = true, int cores =
   return XXp;}
 
 // [[Rcpp::export]]
-Eigen::MatrixXd EigenGAU(Eigen::MatrixXd X, double phi = 1.0, int cores = 2){
-  Eigen::setNbThreads(cores);
+Eigen::MatrixXd EigenGAU(Eigen::MatrixXd X, double phi = 1.0, int cores = 1){
+  if(cores!=1) Eigen::setNbThreads(cores);
   int n = X.rows(); double tmp;
   Eigen::MatrixXd XXp = X*X.transpose();
   for(int i=0; i<n; i++){ for(int j=0; j<n; j++){ if(i>j){
@@ -35,8 +36,9 @@ Eigen::MatrixXd EigenGAU(Eigen::MatrixXd X, double phi = 1.0, int cores = 2){
   XXp *= tmp; return exp(XXp.array());}
 
 // [[Rcpp::export]]
-Eigen::MatrixXd EigenGRM(Eigen::MatrixXd X, bool centralizeZ = true, int cores = 2){
-  Eigen::setNbThreads(cores); int p = X.cols(); double tmp;
+Eigen::MatrixXd EigenGRM(Eigen::MatrixXd X, bool centralizeZ = true, int cores = 1){
+  if(cores!=1) Eigen::setNbThreads(cores); 
+  int p = X.cols(); double tmp;
   if(centralizeZ){
     for(int i=0; i<p; i++){
       tmp = (X.col(i).array()).mean();
@@ -46,38 +48,39 @@ Eigen::MatrixXd EigenGRM(Eigen::MatrixXd X, bool centralizeZ = true, int cores =
   XXp *= tmp; return XXp;}
 
 // [[Rcpp::export]]
-Eigen::MatrixXd EigenCNT(Eigen::MatrixXd X, int cores = 2){
-  Eigen::setNbThreads(cores); int p = X.cols();
+Eigen::MatrixXd EigenCNT(Eigen::MatrixXd X, int cores = 1){
+  if(cores!=1) Eigen::setNbThreads(cores); 
+  int p = X.cols();
   Eigen::VectorXd xx = X.colwise().mean();
   for(int i=0; i<p; i++){ X.col(i) = X.col(i).array() - xx(i);}
   return X;}
 
 // [[Rcpp::export]]
-SEXP EigenEVD(Eigen::MatrixXd A, int cores = 2){
-  Eigen::setNbThreads(cores); 
+SEXP EigenEVD(Eigen::MatrixXd A, int cores = 1){
+  if(cores!=1) Eigen::setNbThreads(cores); 
   Eigen::SelfAdjointEigenSolver<Eigen::MatrixXd> es(A);
   return Rcpp::List::create(Rcpp::Named("U")=es.eigenvectors(),
                             Rcpp::Named("D")=es.eigenvalues());}
 
 // [[Rcpp::export]]
-SEXP EigenBDCSVD(Eigen::MatrixXd X, int cores = 2){
-  Eigen::setNbThreads(cores);
+SEXP EigenBDCSVD(Eigen::MatrixXd X, int cores = 1){
+  if(cores!=1) Eigen::setNbThreads(cores);
   Eigen::BDCSVD<Eigen::MatrixXd> svd(X, Eigen::ComputeThinU | Eigen::ComputeThinV );
   return Rcpp::List::create(Rcpp::Named("U")=svd.matrixU(),
                             Rcpp::Named("D")=svd.singularValues(),
                             Rcpp::Named("V")=svd.matrixV());}
 
 // [[Rcpp::export]]
-SEXP EigenJacobiSVD(Eigen::MatrixXd X, int cores = 2){
-  Eigen::setNbThreads(cores);
+SEXP EigenJacobiSVD(Eigen::MatrixXd X, int cores = 1){
+  if(cores!=1) Eigen::setNbThreads(cores);
   Eigen::JacobiSVD<Eigen::MatrixXd> svd(X, Eigen::ComputeThinU | Eigen::ComputeThinV );
   return Rcpp::List::create(Rcpp::Named("U")=svd.matrixU(),
                             Rcpp::Named("D")=svd.singularValues(),
                             Rcpp::Named("V")=svd.matrixV());}
 
 // [[Rcpp::export]]
-Eigen::VectorXd EigenAcc(Eigen::MatrixXd X1, Eigen::MatrixXd X2, double h2 = 0.5, int cores = 2){
-  Eigen::setNbThreads(cores);
+Eigen::VectorXd EigenAcc(Eigen::MatrixXd X1, Eigen::MatrixXd X2, double h2 = 0.5, int cores = 1){
+  if(cores!=1) Eigen::setNbThreads(cores);
   Eigen::MatrixXd X1X1 = X1*X1.transpose(), X1X2 = X1*X2.transpose();
   double Ve = (1.0-h2)/h2, alpha = 1.0/(X1X1.diagonal().array()).mean();
   Eigen::MatrixXd V = X1X1*alpha; V.diagonal() = V.diagonal().array() + Ve;
